from Paperl.Paperui.Widgets.window import Window
from Paperl.Paperc import prError, prSuccess


class Tooltip(Window):
    def __init__(self, parent, parentWindow, message: str = "", waitTime: int = 1000,
                 hasBorder: bool = True,
                 padding: int = 3,
                 sideType="center",
                 height: int = 40,
                 width=None,
                 cursor: bool = False,
                 background: str = "#ffffff",
                 foreground: str = "#000000"):
        self.build(parent.Me, parentWindow.Me, message, waitTime)

    def build(self, parent, parentWindow, message: str = "", waitTime: int = 1000,
              hasBorder: bool = True,
              padding: int = 3,
              sideType="center",
              height: int = 40,
              width=None,
              cursor: bool = False,
              background: str = "#ffffff",
              foreground: str = "#000000") -> None:
        try:
            from tkdev4 import DevTooltip
        except:
            pass
        else:
            try:
                self.Me = DevTooltip(widget=parent, window=parentWindow, message=message, after=waitTime, border=hasBorder,
                                     padding=padding, side=sideType, height=height, width=width, cursor=cursor,
                                     background=background, foreground=foreground)
            except:
                prError("ToolTip -> Creante -> Error")


class TooltipEx(object):
    def __init__(self, parent, message: str = "", waitTime: float = 0.5):
        self.build(parent.Me, message, waitTime)

    def build(self, parent, message: str = "", waitTime: float = 0.5):
        try:
            from tktooltip import ToolTip
        except:
            prError("tkinter-tooltip -> Check -> Not Installed")
        else:
            prSuccess("tkinter-tooltip -> Check -> Installed")
            self.Me = ToolTip(parent, message, waitTime)


from Paperl.Paperui.Widgets.window import Window
from Paperl.Paperc import prDebugging, prError, prSuccess


class Toplevel(Window):
    __name__ = "Toplevel"

    def __init__(self, parent: Window = None):
        """
        顶级窗口组件，可以直接使用

        ------------------------

        示例

        from Paperl import *

        Application = Application()

        Window = Window()

        Toplevel = Toplevel()

        Application.run(Window)

        """
        if parent is None:
            self.build(None)
        else:
            self.build(parent.Me)
        self.init()
        try:
            from tkdev4 import DevManage
        except:
            pass
        else:
            try:
                self.windows_manage = DevManage(self.Me)
            except:
                pass
        self.setSystemBackdropNone()

    def build(self, parent) -> None:
        from tkinter import Toplevel
        self.Me = Toplevel(parent)
        prDebugging("Toplevel -> Build")



from Paperl.Paperui.Widgets.toplevel import Toplevel
from Paperl.Paperui.Widgets.window import Window


class ChildWindowEx(Toplevel):
    def __init__(self, parent: Window):
        super().__init__(parent)
        self.embedEx(parent)


from Paperl.Paperui.Widgets.widget import Widget


class ResizeHandleIx(object):
    def __init__(self, parent: Widget, handleSize=8, gridded=1, minimizeWidth=50, minimizeHeight=50):
        self.build(parent.Me, handleSize=handleSize, gridded=gridded, minimizeWidth=minimizeWidth, minimizeHeight=minimizeHeight)

    def build(self, parent: Widget, handleSize=8, gridded=1, minimizeWidth=50, minimizeHeight=50):
        from tkinter.tix import ResizeHandle
        self.Me = ResizeHandle(parent, handlesize=handleSize, gridded=gridded, minwidth=minimizeWidth, minheight=minimizeHeight)

    def attach(self, widget: Widget):
        self.Me.attach_widget(widget.Me)

    def detach(self, widget: Widget):
        self.Me.detach_widget(widget.Me)

    def show(self, widget: Widget):
        self.Me.show(widget)

    def hide(self, widget: Widget):
        self.Me.hide(widget)


from Paperl.Paperui.Widgets.widget import Widget


class SpinBox(Widget):
    def __init__(self, parent: Widget, values: list = [""], startValue: float = 0, endValue: float = 10):
        self.build(parent.Me, values, startValue, endValue)

    def build(self, parent, values: list = None, startValue=0, endValue: float = 10):
        from tkinter.ttk import Spinbox
        self.Me = Spinbox(parent, values=values, from_=startValue, to=endValue)



from Paperl.Paperui.Widgets.widget import Widget


class ComboBox(Widget):
    def __init__(self, parent: Widget, values=[]):
        self.build(parent.Me, values)

    def build(self, parent: Widget, values=[]):
        from tkinter import ttk
        self.Me = ttk.Combobox(parent, values=values)

    def current(self, index):
        self.Me.current(index)

    def setReadonly(self):
        self.setState("readonly")

    def setValues(self, values):
        self.Me.configure(values=values)

    def getValues(self):
        return self.Me.cget("values")

    def setText(self, text: str):
        self.Me.set(text)

    def getText(self):
        return self.Me.get()


from Paperl.Paperui.Widgets.widget import Widget


class Frame(Widget):
    def __init__(self, parent: Widget):
        self.build(parent.Me)

    def build(self, parent: Widget):
        from tkinter.ttk import Frame
        self.Me = Frame(parent)


from Paperl.Paperui.Widgets.widget import Widget


class Entry(Widget):
    def __init__(self, parent: Widget, text: str = ""):
        self.build(parent.Me, text)

    def build(self, parent: Widget, text: str = ""):
        from tkinter.ttk import Entry
        from tkinter import StringVar
        self.Var = StringVar()
        self.Var.set(text)
        self.Me = Entry(parent, textvariable=text)

    def setText(self, text: str):
        self.Var.set(text)

    def getText(self):
        return self.Var.get()



from Paperl.Paperui.Widgets.widget import Widget
from Paperl.Paperui.Widgets.stringsVar import StringsVar


class OptionMenu(Widget):
    def __init__(self, parent: Widget, defaultValue=None, valueList=[""]):
        self.build(parent.Me, defaultValue, valueList)

    def build(self, parent: Widget, defaultValue: str = None, valueList=[""]):
        from tkinter.ttk import OptionMenu
        self.Var = StringsVar()
        self.Me = OptionMenu(parent, self.Var.Me, defaultValue, *valueList)

    def setList(self, defaultValue=None, valueList=[""]):
        self.Me.set_menu(defaultValue, *valueList)

    def setMenu(self, defaultValue=None, valueList=[""]):
        self.Me.set_menu(defaultValue, *valueList)

    def setValue(self, value: str):
        self.Var.setValue(value)

    def getValue(self):
        return self.Var.getValue()



from Paperl.Paperui.Widgets.widget import Widget


class ColorChooserDialog(object):
    def __init__(self):
        self.build()

    def build(self):
        pass

    def getColor(self):
        from tkinter.colorchooser import askcolor
        return askcolor()

    def getColorRgb(self):
        return self.getColor()[0]

    def getColorHex(self):
        return self.getColor()[1]

def getColor():
    return ColorChooserDialog().getColor()

def getColorRgb():
    return ColorChooserDialog().getColorRgb()

def getColorHex():
    return ColorChooserDialog().getColorHex()


class Drag(object):
    def __init__(self, parentWidget, dragWidget, dragX: bool = True, dragY: bool = True, isWindow: bool = False):
        self.build(parentWidget, dragWidget, dragX, dragY, isWindow)

    def build(self, parentWidget, dragWidget, dragX: bool = True, dragY: bool = True, isWindow: bool = False):
        from tkdev4 import DevDrag
        self.Me = DevDrag(widget=parentWidget.Me, dragwidget=dragWidget.Me, x=dragX, y=dragY, iswindow=isWindow)


from Paperl.Paperui.Widgets.widget import Widget


class StatusBar(Widget):
    def __init__(self, parent: Widget):
        self.build(parent.Me)

    def build(self, parent):
        from tkdev4 import DevStatusBar
        self.Me = Sizegrip(parent)



from Paperl.Paperui.Widgets.widget import Widget


class Animation(object):
    def __init__(self, control: Widget):
        self.Me = control

    def gradualMovement(self, action, move: int = 100, time: float = 0.01):
        try:
            def motion():
                try:
                    from time import sleep
                    for c in range(move):
                        sleep(time)
                        if action == "left-right":
                            self.Me.place(x=self.Me.getPositionX() + 1, y=self.Me.getPositionY())
                        elif action == "right-left":
                            self.Me.place(x=self.Me.getPositionX() - 1, y=self.Me.getPositionY())
                        elif action == "top-bottom":
                            self.Me.place(x=self.Me.getPositionX(), y=self.Me.getPositionY() + 1)
                        elif action == "bottom-top":
                            self.Me.place(x=self.Me.getPositionX(), y=self.Me.getPositionY() - 1)
                        self.Me.upDate()
                except:
                    pass
            self.Me.waitEvent(10, motion)
        except:
            pass

    def gradualWindowShow(self, number: int = 100, time: float = 0.01):
        self.Me.setAlpha(0.0)
        from time import sleep
        for num in range(number):
            sleep(time)
            self.Me.setAlpha(num/100)
            self.Me.upDate()
        self.Me.setAlpha(1.0)

    def gradualWindowHide(self, number: int = 100, time: float = 0.01):
        self.Me.setAlpha(1.0)
        from time import sleep
        for num in range(number):
            sleep(time)
            self.Me.setAlpha(1-num/100)
            self.Me.upDate()
        self.Me.setAlpha(0.0)



from Paperl.Paperc import prWarring, prSuccess, prError
from typing import Literal


class Themes(object):
    def getSystemStyles(self):
        """
        获取到系统所有的主题名称

        :return: 系统所有的主题
        """

        from tkinter.ttk import Style
        return Style().theme_names()

    def useStyle(self, styleName):
        """
        启用系统主题（无需安装库）

        :param styleName: 系统主题的名称
        """
        try:
            from tkinter.ttk import Style
            Style().theme_use(styleName)
        except:
            prWarring(f"Widget -> Style -> {styleName} -> This system does not support this theme")

    def useStyleAqua(self):
        """
        启用系统Aqua主题（无需安装库）
        """
        self.useStyle("aqua")

    def useStyleVista(self):
        """
        启用系统Vista主题（无需安装库）
        """
        self.useStyle("vista")

    def useStyleWinNative(self):
        """
        启用系统WinNative主题（无需安装库）
        """
        self.useStyle("winnative")

    def useStyleDefault(self):
        """
        启用系统默认主题（无需安装库）
        """
        self.useStyle("default")

    def useStyleClassic(self):
        """
        启用系统Classic全系统通用主题（无需安装库）
        """
        self.useStyle("classic")

    def useStyleAlt(self):
        """
        启用系统Alt主题（无需安装库）
        """
        self.useStyle("alt")

    def useStyleClam(self):
        """
        启用系统Clam主题（无需安装库）
        """
        self.useStyle("clam")

    def useStyleSunValley(self, theme: Literal["light", "dark", "auto"] = "auto"):
        """
        启用太阳谷主题（需安装sv-ttk库）（如果需要打包，请从sv-ttk的Github存储库内找到sv_ttk文件夹，然后下载放到程序目录）

        :param theme: 设置太阳谷主题 参考值 -> "light" "dark"
        """
        try:
            from sv_ttk import use_dark_theme, use_light_theme
        except:
            prWarring("sv_ttk -> Check -> Not Installed")
        else:
            def light():
                use_light_theme()
                self.setLightTheme()
                self.setBackground("#fafafa")

            def dark():
                use_dark_theme()
                self.setDarkTheme()
                self.setBackground("#1c1c1c")

            prSuccess("sv_ttk -> Check -> Installed")
            if theme == "auto":
                try:
                    from darkdetect import isDark
                except:
                    prError("darkdetect -> Check -> Not Installed")
                else:
                    if isDark():
                        dark()
                    else:
                        light()
            elif theme == "dark":
                dark()
            else:
                light()

    def useStyleEx(self, styleName):
        """
        启用扩展主题（需安装ttkthemes库）

        :param styleName: 扩展主题的名称
        """
        try:
            from ttkthemes import ThemedStyle
        except:
            prWarring("ttkthemes -> Check -> Not Installed")
        else:
            prSuccess("ttkthemes -> Check -> Installed")
            try:
                ThemedStyle(self.Me).theme_use(styleName)
            except:
                prError(f"ttkthemes -> {styleName} -> The theme could not be found")

    def useStyleExArc(self):
        """
        启用扩展Arc主题（需安装ttkthemes库）
        """
        self.useStyleEx("arc")

    def useStyleExEquilux(self):
        """
        启用扩展Equilux主题（需安装ttkthemes库）
        """
        self.useStyleEx("equilux")

    def useStyleExWinXpBlue(self):
        """
        启用扩展WinXpBlue主题（需安装ttkthemes库）
        """
        self.useStyleEx("winxpblue")

    def useStyleExAquativo(self):
        """
        启用扩展Aquativo主题（需安装ttkthemes库）
        """
        self.useStyleEx("aquativo")

    def useStyleExAdapta(self):
        """
        启用扩展Adapta主题（需安装ttkthemes库）
        """
        self.useStyleEx("adapta")

    def useStyleExBreeze(self):
        """
        启用扩展Breeze主题（需安装ttkthemes库）
        """
        self.useStyleEx("breeze")

    def useStyleExRadiance(self):
        """
        启用扩展Radiance主题（需安装ttkthemes库）
        """
        self.useStyleEx("radiance")

    def useStyleExYaru(self):
        """
        启用扩展Yaru主题（需安装ttkthemes库）
        """
        self.useStyleEx("yaru")


from Paperl.Paperui.Widgets.widget import Widget


class MenuButton(Widget):
    def __init__(self, parent: Widget, text: str = "", menu: Widget = None):
        if menu is None:
            Menu = None
        else:
            Menu = menu.Me
        self.build(parent.Me, text, menu)

    def build(self, parent: Widget, text: str = "", menu: Widget = None):
        from tkinter.ttk import Menubutton
        self.Me = Menubutton(parent, text=text, menu=menu)



def getText(widget):
    try:
        from win32gui import GetWindowText
    except:
        pass
    else:
        try:
            return GetWindowText(widget.gethWnd())
        except:
            pass

def setText(widget, text: str = ""):
    try:
        from win32gui import SetWindowText
    except:
        pass
    else:
        try:
            return SetWindowText(widget.gethWnd(), text)
        except:
            pass


class StringsVar(object):
    def __init__(self):
        self.build()

    def build(self):
        from tkinter import StringVar
        self.Me = StringVar()

    def setValue(self, value):
        self.Me.set(value)

    def getValue(self):
        return self.Me.get()


from Paperl.Paperui.Widgets.window import Window
from Paperl.Paperc import prDebugging


class WindowIx(Window):
    def __init__(self):
        super().__init__()

    def build(self) -> None:
        from tkinter import tix
        self.Me = tix.Tk()
        prDebugging("WindowIX -> Build")



from Paperl.Paperui.Widgets.widget import Widget


class Menu(Widget):
    def __init__(self, parent: Widget = None, title: str = ""):
        if parent is None:
            Parent = None
        else:
            Parent = parent.Me
        self.build(Parent, title)

    def build(self, parent: Widget, title: str = ""):
        from tkinter import Menu
        self.Me = Menu(parent, title=title, border=0)
        self.setTearOff(False)

    def addCommand(self, label: str = "", state="normal",):
        self.Me.add_command(label=label, state=state)

    def onTearOff(self, eventFunc):
        self.Me.configure(tearoffcommand=eventFunc)

    def addMenu(self, menu: Widget, label: str = "", state="normal"):
        self.Me.add_cascade(label=label, state=state, menu=menu.Me)


from Paperl.Paperui.Widgets.widget import Widget


class ProgressBar(Widget):
    def __init__(self, parent: Widget, orient="horizontal", mode="determinate", length=100, value: float = 0, maxValue: float = 100):
        self.build(parent.Me, orient, mode, length, value, maxValue)

    def build(self, parent: Widget, orient, mode, length, value, maxValue):
        from tkinter.ttk import Progressbar
        self.Me = Progressbar(parent, orient=orient, mode=mode, length=length, value=value, maximum=maxValue)

    def setLength(self, length):
        self.Me.configure(length=length)

    def getLength(self):
        return self.Me.cget("length")

    def setOrient(self, orient):
        self.Me.configure(orient=orient)

    def getOrient(self):
        return self.Me.cget("orient")

    def setValue(self, value):
        self.Me.configure(value=value)

    def getValue(self):
        return self.Me.cget("value")

    def setMaxValue(self, value):
        self.Me.configure(maximum=value)

    def getMaxValue(self):
        return self.Me.cget("maximum")

    def startProgress(self, time: int = 50):
        self.Me.start(time)

    def stepProgress(self, value: float = 1.0):
        self.Me.step(value)

    def stopProgress(self):
        self.Me.stop()


from Paperl.Paperui.Widgets.widget import Widget


class Message(Widget):
    def __init__(self, parent: Widget, text: str = ""):
        self.build(parent.Me, text)

    def build(self, parent: Widget, text: str = ""):
        from tkinter import Message
        self.Me = Message(parent, text=text, anchor="nw")



from Paperl.Paperui.Widgets.frame import Frame
from Paperl.Paperui.Widgets.widget import Widget


class FrameIx(Frame):
    def __init__(self, parent: Widget):
        self.build(parent.Me)

    def build(self, parent: Widget):
        from tkinter.tix import Frame
        self.Me = Frame(parent)



from Paperl.Paperui.Widgets.widget import Widget


class Group(Widget):
    def __init__(self, parent: Widget, label: str = "", widget: Widget = None, anchor: str = "nw"):
        self.build(parent.Me, label, widget, anchor)

    def build(self, parent: Widget, label: str = "", widget: Widget = None, anchor: str = "nw"):
        from tkinter.ttk import Labelframe
        if widget is None:
            self.Me = Labelframe(parent, text=label)
        else:
            self.Me = Labelframe(parent, labelwidget=widget.Me, labelanchor=anchor)

    def setWidget(self, widget: Widget = None, anchor: str = "nw"):
        self.Me.configure(labelwidget=widget, labelanchor=anchor)


from Paperl.Paperui.Widgets.widget import Widget


class Label(Widget):
    def __init__(self, parent: Widget, text: str = ""):
        self.build(parent.Me, text)

    def build(self, parent: Widget, text: str = ""):
        from tkinter.ttk import Label
        self.Me = Label(parent, text=text)



import sys
from functools import singledispatch

try:
    import darkdetect
except:
    pass

from Paperl.Paperc import prWarring, prSuccess, prError
from typing import Literal

try:
    from ctypes import Structure, c_int, POINTER, pointer, sizeof
    from ctypes.wintypes import DWORD, ULONG
    import ctypes
except:
    pass
else:
    class AccentPolicy(Structure):
        """ 设置客户区的具体属性 """
        _fields_ = [
            ('AccentState', DWORD),
            ('AccentFlags', DWORD),
            ('GradientColor', DWORD),
            ('AnimationId', DWORD),
        ]


    class WindowCompositionAttribute(Structure):
        _fields_ = [
            ('Attribute', DWORD),
            ('Data', POINTER(AccentPolicy)),  # POINTER()接收任何ctypes类型，并返回一个指针类型
            ('SizeOfData', ULONG),
        ]

if sys.platform == "win32":
    try:
        import win32gui
    except:
        pass
    else:
        pass


        class ExButton(object):
            def __init__(self, parent, text: str = "", pos=(10, 10), size=(100, 100), className="Button"):
                self.initEvent()
                self.build(parent, text, pos, size, className)

            def build(self, parent, text: str = "", pos=(10, 10), size=(100, 100), className="Button"):
                from win32gui import WNDCLASS, CreateWindow, RegisterClass
                from win32con import COLOR_BTNFACE, WS_VISIBLE, WS_CHILD, BS_PUSHBUTTON, WS_TABSTOP
                wnd = WNDCLASS()
                wnd.lpszClassName = className
                wnd.hbrBackground = COLOR_BTNFACE + 1
                wnd.lpfnWndProc = self.windowProc
                classAtom = RegisterClass(wnd)
                self.hWnd = CreateWindow(
                    classAtom, text, WS_VISIBLE | WS_CHILD | BS_PUSHBUTTON | WS_TABSTOP,
                    pos[0], pos[1],
                    size[0], size[1],
                    parent.gethWnd(), None, parent.gethWnd(), None)

            def initEvent(self):
                self.eventDestroy = self.quit

            def quit(self):
                from win32gui import PostQuitMessage
                PostQuitMessage(0)

            def onDestroy(self, eventFunc: None = ...):
                self.eventDestroy()

            def windowProc(self, hWnd, message, wParam, lParam):
                if message == WM_DESTROY:
                    self.eventDestroy()
                return DefWindowProc(hWnd, message, wParam, lParam)


class Windows22H2(object):
    def setTransient(self):
        self.setSystemBackdropTransientWindow()
        self.setDarkTheme()
        self.setBackground("#000000")
        self.setExtendFrameIntoClientArea(-1, -1, -1, -1)

    def setSystemBackdropNone(self):
        try:
            self.windows_manage.dwm_set_window_attribute_systembackdrop_type_none()
        except:
            pass

    def setSystemBackdropAuto(self):
        try:
            self.windows_manage.dwm_set_window_attribute_systembackdrop_type_auto()
        except:
            pass

    def setSystemBackdropMainWindow(self):
        try:
            self.windows_manage.dwm_set_window_attribute_systembackdrop_type_mainwindow()
        except:
            pass

    def setSystemBackdropTabbedWindow(self):
        try:
            self.windows_manage.dwm_set_window_attribute_systembackdrop_type_tabbed_window()
        except:
            pass

    def setSystemBackdropTransientWindow(self):
        try:
            self.windows_manage.dwm_set_window_attribute_systembackdrop_type_transient_window()
        except:
            pass


class Windows21H2(object):
    def setAutoTheme(self):
        try:
            import darkdetect
            if darkdetect.isDark():
                self.windows_manage.dwm_set_window_attribute_use_dark_mode()
            else:
                self.windows_manage.dwm_set_window_attribute_use_light_mode()
        except:
            pass

    def setDarkTheme(self):
        try:
            self.windows_manage.dwm_set_window_attribute_use_dark_mode()
        except:
            pass

    def setLightTheme(self):
        try:
            self.windows_manage.dwm_set_window_attribute_use_light_mode()
        except:
            pass

    def useMica(self, theme: Literal["light", "dark", "auto"] = "auto"):
        """
        启用云母特效（仅支持windows11）（需安装win32mica库）

        :param theme: 设置云母特效的主题 参考值 -> "light" "dark"
        """
        import sys
        if sys.platform == "win32" and sys.getwindowsversion().build >= 22000:
            try:
                from win32mica import ApplyMica
            except:
                prWarring("win32mica -> Check -> Not Installed")
            else:
                try:
                    def light():
                        ApplyMica(self.gethWnd(), False)
                        self.setBackground("#fcfcfc")

                    def dark():
                        ApplyMica(self.gethWnd(), True)
                        self.setBackground("#000000")

                    if theme == "auto":
                        try:
                            from darkdetect import isDark
                        except:
                            prError("darkdetect -> Check -> Not Installed")
                        else:
                            if isDark():
                                dark()
                            else:
                                light()
                    elif theme == "dark":
                        dark()
                    else:
                        light()
                except:
                    prWarring("win32mica -> Use -> Cannot be used normally")

    def setCaptionColor(self, hex: str):
        try:
            self.windows_manage.dwm_set_window_attribute_caption_color(self.windows_manage.get_hex(hex))
        except:
            pass

    def setBorderColor(self, hex: str):
        try:
            self.windows_manage.dwm_set_window_attribute_border_color(self.windows_manage.get_hex(hex))
        except:
            pass

    def setTitleColor(self, hex: str):
        try:
            self.windows_manage.dwm_set_window_attribute_text_color(self.windows_manage.get_hex(hex))
        except:
            pass

    def setRound(self):
        try:
            self.windows_manage.dwm_set_window_round_round()
        except:
            pass

    def setRoundSmall(self):
        try:
            self.windows_manage.dwm_set_window_round_round_small()
        except:
            pass

    def setRoundDoNot(self):
        try:
            self.windows_manage.dwm_set_window_round_donot_round()
        except:
            pass

    def setExtendFrameIntoClientArea(self, Left, Right, Top, Bottom):
        try:
            self.windows_manage.dwm_extend_frame_into_client_area((Left, Right, Top, Bottom))
        except TypeError:
            prWarring("Window -> TypeError -> Cannt set ExtendFrameIntoClientArea")


class WindowsAll(object):
    def registerThumbnail(self):
        try:
            import ctypes
            from win32con import NULL
            from win32gui import FindWindow
            self.RegisterThumbnail = ctypes.windll.dwmapi.DwmRegisterThumbnail
            id = self.RegisterThumbnail(self.gethWnd(),
                                        FindWindow("Program", None),
                                        None)
            if id == -2147024809:
                return False
            else:
                return True
        except:
            return None

    def getColorizationColor(self):
        try:
            import ctypes
            from win32con import NULL
            from win32gui import FindWindow
            self.GetColorizationColor = ctypes.windll.dwmapi.DwmGetColorizationColor
            color = self.GetColorizationColor()
            if color == -2147024809:
                return False
            else:
                return color
        except:
            return None


class WindowsEffect(object):
    def __init__(self):
        self.ACCENT_DISABLED = 0,
        self.ACCENT_ENABLE_GRADIENT = 1,
        self.ACCENT_ENABLE_TRANSPARENTGRADIENT = 2,
        self.ACCENT_ENABLE_BLURBEHIND = 3,  # Aero效果
        self.ACCENT_ENABLE_ACRYLICBLURBEHIND = 4,  # 亚克力效果
        self.ACCENT_INVALID_STATE = 5

        self.WCA_UNDEFINED = 0,
        self.WCA_NCRENDERING_ENABLED = 1,
        self.WCA_NCRENDERING_POLICY = 2,
        self.WCA_TRANSITIONS_FORCEDISABLED = 3,
        self.WCA_ALLOW_NCPAINT = 4,
        self.WCA_CAPTION_BUTTON_BOUNDS = 5,
        self.WCA_NONCLIENT_RTL_LAYOUT = 6,
        self.WCA_FORCE_ICONIC_REPRESENTATION = 7,
        self.WCA_EXTENDED_FRAME_BOUNDS = 8,
        self.WCA_HAS_ICONIC_BITMAP = 9,
        self.WCA_THEME_ATTRIBUTES = 10,
        self.WCA_NCRENDERING_EXILED = 11,
        self.WCA_NCADORNMENTINFO = 12,
        self.WCA_EXCLUDED_FROM_LIVEPREVIEW = 13,
        self.WCA_VIDEO_OVERLAY_ACTIVE = 14,
        self.WCA_FORCE_ACTIVEWINDOW_APPEARANCE = 15,
        self.WCA_DISALLOW_PEEK = 16,
        self.WCA_CLOAK = 17,
        self.WCA_CLOAKED = 18,
        self.WCA_ACCENT_POLICY = 19,
        self.WCA_FREEZE_REPRESENTATION = 20,
        self.WCA_EVER_UNCLOAKED = 21,
        self.WCA_VISUAL_OWNER = 22,
        self.WCA_LAST = 23

    def useAcrylic(self, theme="light"):
        try:
            from BlurWindow.blurWindow import GlobalBlur
        except:
            prWarring("BlurWindow -> Check -> Not Installed")
        else:
            prSuccess("BlurWindow -> Check -> Installed")
            try:
                if theme == "dark":
                    GlobalBlur(self.gethWnd(), Dark=True)
                else:
                    GlobalBlur(self.gethWnd(), Dark=False)
            except:
                prWarring("BlurWindow -> Use -> Cannot be used normally")

    def setWindowCompositionAttribute(self, Attribute):
        try:
            self.SetWindowCompositionAttribute(self.gethWnd(), Attribute)
        except:
            prError("Widget -> SetWindowCompositionAttribute -> Error")

    def setAccent(self, Accent):
        try:
            self.AccentPolicy = AccentPolicy()
            self.AccentPolicy.AccentState = Accent
        except:
            pass
        try:
            import ctypes
            self.WindowCompAttributeData = WindowCompositionAttribute()
            self.WindowCompAttributeData.Attribute = 19
            self.WindowCompAttributeData.SizeOfData = sizeof(self.AccentPolicy)
            self.WindowCompAttributeData.Data = pointer(self.AccentPolicy)
            ctypes.windll.user32.SetWindowCompositionAttribute(self.gethWnd(), pointer(self.WindowCompAttributeData))
        except TypeError:
            prError("Widget -> SetAccent -> Error")

    def useEnableGradient(self):
        self.setAccent(1)

    def useEnableTransparentGradient(self):
        self.setAccent(2)

    def useEnableBlurBehind(self):
        self.setAccent(3)

    def useEnableAcrylicBlurBehind(self):
        self.setAccent(4)

    def useAero(self):
        self.useEnableBlurBehind()


class WindowsDev(Windows21H2, Windows22H2, WindowsEffect, WindowsAll):
    pass



# State
STATE_NORMAL = "normal"
STATE_ACTIVE = "active"
STATE_DISABLED = "disabled"

# Fill
FILL_NONE = "none"
FILL_WIDTH = "x"
FILL_HEIGHT = "y"
FILL_BOTH = "both"

# Expand
EXPAND_NO = "no"
EXPAND_YES = "yes"

# Side
SIDE_LEFT = "left"
SIDE_RIGHT = "right"
SIDE_TOP = "top"
SIDE_BOTTOM = "bottom"

# Anchor
ANCHOR_NW = "nw"
ANCHOR_N = "n"
ANCHOR_NE = "ne"
ANCHOR_W = "w"
ANCHOR_CENTER = "center"
ANCHOR_E = "e"
ANCHOR_SW = "sw"
ANCHOR_S = "s"
ANCHOR_SE = "se"

# Justify
JUSTIFY_LEFT = "left"
JUSTIFY_CENTER = "center"
JUSTIFY_RIGHT = "right"

# Style
STYLE_LIGHT = "light"
STYLE_DARK = "dark"
STYLE_AUTO = "auto"

# Button
EVENT_BUTTON1 = "<Button-1>"
EVENT_BUTTON2 = "<Button-2>"
EVENT_BUTTON3 = "<Button-3>"
EVENT_BUTTON_ALL = "<Button>"
EVENT_BUTTON_PRESS = "<ButtonPress>"

# Button Release
EVENT_BUTTON1_RELEASE = "<ButtonRelease-1>"
EVENT_BUTTON2_RELEASE = "<ButtonRelease-2>"
EVENT_BUTTON3_RELEASE = "<ButtonRelease-3>"
EVENT_BUTTON_ALL_RELEASE = "<ButtonRelease>"

# Double Button
EVENT_BUTTON1_DOUBLE = "<Double-Button-1>"
EVENT_BUTTON2_DOUBLE = "<Double-Button-2>"
EVENT_BUTTON3_DOUBLE = "<Double-Button-3>"
EVENT_BUTTON_ALL_DOUBLE = "<Double-Button>"
EVENT_BUTTON_PRESS_DOUBLE = "<Double-ButtonPress>"

# Triple Button
EVENT_BUTTON1_TRIPLE = "<Triple-Button-1>"
EVENT_BUTTON2_TRIPLE = "<Triple-Button-2>"
EVENT_BUTTON3_TRIPLE = "<Triple-Button-3>"
EVENT_BUTTON_ALL_TRIPLE = "<Triple-Button>"
EVENT_BUTTON_PRESS_TRIPLE = "<Triple-ButtonPress>"

# Event
EVENT_ENTER = "<Enter>"
EVENT_LEAVE = "<Leave>"

EVENT_CONFIGURE = "<Configure>"

EVENT_DESTROY = "<Destroy>"

EVENT_FOCUS_IN = "<FocusIn>"
EVENT_FOCUS_OUT = "<FocusOut>"

# Event Ex
EVENT_EX_DELETE_WINDOW = "WM_DELETE_WINDOW"
EVENT_EX_DELETE_WINDOW = "WM_SAVE_YOURSELF"

ATTRIBUTE_ALPHA = "-alpha"
ATTRIBUTE_TOPPING = "-topmost"
ATTRIBUTE_MODIFIED = "-modified"
ATTRIBUTE_TOOLWINDOW = "-toolwindow"
ATTRIBUTE_TRANSPARENT = "-transparent"

# Orient
ORIENT_HORIZONTAL = "horizontal"
ORIENT_VERTICAL = "vertical"

# Progress Mode
PROGRESS_MODE_DETERMINATE = "determinate"
PROGRESS_MODE_INDETERMINATE = "indeterminate"


from Paperl.Paperui.Widgets.widget import Widget
from Paperl.Paperui.Widgets.button import Button
from Paperl.Paperui.Widgets.booleanVar import BooleanVar


class CheckButton(Button):
    def __init__(self, parent: Widget, text: str = "", falseVar=0, trueVar=1):
        self.build(parent.Me, text=text, falseVar=falseVar, trueVar=trueVar)

    def build(self, parent: Widget, text: str = "", falseVar=0, trueVar=1):
        from tkinter.ttk import Checkbutton
        self.Var = BooleanVar()
        self.Var.setValue(0)
        self.Me = Checkbutton(parent, text=text, variable=self.Var.Me, offvalue=falseVar, onvalue=trueVar)

    def inVoke(self):
        self.Me.invoke()

    def setValue(self, value: bool):
        self.Var.setValue(value)

    def getValue(self):
        return self.Var.getValue()


from Paperl.Paperui.Extension.selectedCanvas import SelectedCanvas
from Paperl.Paperui.Extension.balloon import Balloon
from Paperl.Paperui.Extension.colorChooser import ColorChooserDialog, getColor, getColorRgb, getColorHex

from Paperl.Paperui.Extension.windowIX import WindowIx
from Paperl.Paperui.Extension.frameIX import FrameIx
from Paperl.Paperui.Extension.resizeHandleIX import ResizeHandleIx


from colorama import Fore


def prDebug(strings: str):
    print(Fore.LIGHTBLUE_EX + f"[DEBUG] {strings}" + Fore.RESET)

def prDebugging(strings: str):
    print(Fore.BLUE + f"[DEBUGGING] {strings}" + Fore.RESET)

def prWarring(strings: str) -> None:
    print(Fore.LIGHTYELLOW_EX + "[WARRING] " + strings + Fore.RESET)

def prError(strings: str) -> None:
    print(Fore.LIGHTRED_EX + "[ERROR] " + strings + Fore.RESET)

def prSuccess(strings: str) -> None:
    print(Fore.LIGHTGREEN_EX + "[SUCCESS] " + strings + Fore.RESET)

def checkSystem():
    import sys
    import platform
    if sys.platform == "win32":
        prDebug("System -> Window")
        try:
            import tkdev4
        except:
            prWarring("tkDev4 -> Check -> Not Installed")
        else:
            prSuccess("tkDev4 -> Check -> Installed")
            try:
                import win32gui
            except:
                prError("tkDev4 -> Use -> Cannot be used normally")
            else:
                prSuccess("tkDev4 -> Use -> Can be used normally")
    elif sys.platform == "darwin":
        prDebug("System -> macOS")
    elif sys.platform == "linux":
        prDebug("System -> Linux")
    else:
        prError("System -> Get -> Unrecognized")
    prDebug(f"System.PlatForm -> {platform.platform()}")
    prDebug(f"System.Versions -> {platform.version()}")

def checkPython():
    import platform
    prDebug(f"Python.Versions -> {platform.python_version()}")
    prDebug(f"Python.Build -> {platform.python_build()}")


from Paperl.Paperui.Widgets.widget import Widget
from Paperl.Paperui.Widgets.windowDevelop import WindowsDev
from Paperl.Paperui.Widgets.themes import Themes
from Paperl.Paperc import prDebugging, prError, prSuccess, prWarring
from functools import singledispatch
from typing import Literal


class Window(Widget, WindowsDev, Themes):
    __name__ = "Window"

    def __init__(self):
        """
        窗口组件，可以配合Application一起使用，也可以直接使用

        ------------------------

        示例

        from Paperl import *

        Application = Application()

        Window = Window()

        Application.run(Window)

        ------------------------

        示例

        from Paperl import *

        Window = Window()

        Window.run()

        """

        self.build()
        self.init()
        try:
            from tkdev4 import DevManage
        except:
            pass
        else:
            try:
                self.windows_manage = DevManage(self.Me)
            except:
                pass
        self.setSystemBackdropNone()

    def build(self) -> None:
        from tkinter import Tk
        self.Me = Tk()
        prDebugging("Window -> Build")

    def bindEventEx(self, eventName=None, eventFunc: None = ...):
        self.Me.protocol(eventName, eventFunc)

    def jumpTaskBar(self):
        from ctypes import windll

        GWL_EXSTYLE = -20
        WS_EX_APPWINDOW = 0x00040000
        WS_EX_TOOLWINDOW = 0x00000080

        style = windll.user32.GetWindowLongW(self.gethWnd(), GWL_EXSTYLE)
        style = style & ~WS_EX_TOOLWINDOW
        style = style | WS_EX_APPWINDOW
        res = windll.user32.SetWindowLongW(self.gethWnd(), GWL_EXSTYLE, style)
        # re-assert the new window style
        self.hide()
        self.waitEvent(10, lambda: self.show())

    def createHeaderBar(self, border: bool = True, sizeGrip: bool = False):
        from Paperl.Paperui.Widgets.headerBar import HeaderBarEx
        HeaderBar = HeaderBarEx(self, self, border=border)
        HeaderBar.addTitleBar()
        HeaderBar.setTitle(self.getTitle())
        HeaderBar.useDoubleLeftMaximize()
        HeaderBar.useDoubleMiddleClose()
        if sizeGrip:
            HeaderBar.addSizeGrip()
        HeaderBar.show()
        return HeaderBar

    def canResize(self, width: bool = None, height: bool = None):
        self.Me.resizable(width, height)

    def setMinsize(self, width: int = None, height: int = None):
        self.Me.minsize(width, height)

    def setMaxsize(self, width: int = None, height: int = None):
        self.Me.maxsize(width, height)

    def setWidgetIsWindow(self, widget):
        self.Me.manage(widget.Me)

    def minimize(self):
        self.Me.iconify()

    def normal(self):
        self.Me.deiconify()

    def onSaveYourself(self, eventFunc: None = ...):
        self.bindEventEx("WM_SAVE_YOURSELF", eventFunc)

    def onDeleteWindow(self, eventFunc: None = ...):
        self.bindEventEx("WM_DELETE_WINDOW", eventFunc)

    def setPalette(self, *args, **kwargs):
        self.Me.tk_setPalette(args, kwargs)

    def setAttribute(self, attributeName, attributeValue=True):
        try:
            self.Me.attributes(attributeName, attributeValue)
        except:
            prWarring("Window -> SetAttribute -> The system does not support this property")

    def popup(self, x, y):
        self.setPosition(x, y)

    def setAlpha(self, value: float = 1.0):
        self.setAttribute("-alpha", value)

    def setModified(self, bool: bool = True):
        self.setAttribute("-modified", bool)

    def setTransparent(self, bool: bool = True):
        self.setAttribute("-transparent", bool)

    def setTopping(self, isTop: bool = True):
        self.setAttribute("-topmost", isTop)

    def setToolWindow(self, isToolWindow: bool = True):
        self.setAttribute("-toolwindow", isToolWindow)

    def bell(self):
        self.Me.bell()

    def init(self) -> None:
        self.setEmptyIcon()
        self.setBackground("#ffffff")
        self.setTitle("Paperl")
        self.setSize(250, 250)

    def setIcon(self, image):
        self.Me.iconbitmap(image)

    def setEmptyIcon(self):
        try:
            from tkdev4.devicon import Icon_Empty
            self.emptyIcon = Icon_Empty
            self.setIcon(Icon_Empty)
        except:
            pass

    def setFolderIcon(self):
        try:
            from tkdev4.devicon import Icon_Folder
            self.folderIcon = Icon_Folder
            self.setIcon(Icon_Folder)
        except:
            pass

    def setPythonIcon(self):
        try:
            from sys import executable
            from os.path import exists, split
            icon = split(executable)[0] + "\\DLLs\\py.ico"
            if exists(icon):
                self.pythonIcon = icon
                self.setIcon(icon)
        except:
            pass

    def setPythoncIcon(self):
        try:
            from sys import executable
            from os.path import exists, split
            icon = split(executable)[0] + "\\DLLs\\pyc.ico"
            if exists(icon):
                self.pythoncIcon = icon
                self.setIcon(icon)
        except:
            pass

    def setPythondIcon(self):
        try:
            from sys import executable
            from os.path import exists, split
            icon = split(executable)[0] + "\\DLLs\\pyd.ico"
            if exists(icon):
                self.pythondIcon = icon
                self.setIcon(icon)
        except:
            pass

    def setIdleIcon(self):
        try:
            from sys import executable
            from os.path import exists, split
            icon = split(executable)[0] + "\\Lib\\idlelib\\Icons\\idle.ico"
            if exists(icon):
                self.pythondIcon = icon
                self.setIcon(icon)
        except:
            pass

    def setText(self, text: str):
        from tkinter import TclError
        try:
            self.setTitle(text)
        except TclError:
            prError("Window -> Text -> This property is not supported or this value is not supported")

    def getText(self):
        from tkinter import TclError
        try:
            return self.getTitle()
        except TclError:
            prError("Window -> Text -> This property is not supported or this value is not supported")

    def setTitle(self, title: str) -> None:
        self.Me.title(title)

    def getTitle(self) -> str:
        return self.Me.title()

    def setGeometry(self, width: int, height: int, x: int, y: int) -> None:
        self.Me.geometry(f"{str(width)}x{str(height)}+{str(x)}+{str(y)}")

    def getGeometry(self):
        return self.getSize(), self.getPosition()

    def setSize(self, width: int, height: int) -> None:
        self.Me.geometry(f"{str(width)}x{str(height)}")

    def setPosition(self, x: int, y: int) -> None:
        self.Me.geometry(f"+{str(x)}+{str(y)}")

    def maximizeBox(self):
        try:
            self.windows_manage.add_window_maximizebox()
        except:
            pass

    def minimizeBox(self):
        try:
            self.windows_manage.add_window_minimizebox()
        except:
            pass

    def systemMenu(self):
        try:
            self.windows_manage.add_window_sysmenu()
        except:
            pass

    def showWindow(self, nCmdShow):
        try:
            from win32gui import ShowWindow
        except:
            pass
        else:
            ShowWindow(self.gethWnd(), nCmdShow)

    def showMaximizeEx(self):
        try:
            from win32con import SW_SHOWMAXIMIZED
        except:
            pass
        else:
            self.showWindow(SW_SHOWMAXIMIZED)

    def showMaximizeLx(self):
        self.setState("zoomed")

    def showMinimizeEx(self):
        try:
            from win32con import SW_SHOWMINIMIZED
        except:
            pass
        else:
            self.showWindow(SW_SHOWMINIMIZED)

    def showNormalEx(self):
        try:
            from win32con import SW_SHOWNORMAL
        except:
            pass
        else:
            self.showWindow(SW_SHOWNORMAL)

    def hideEx(self):
        try:
            from win32con import SW_HIDE
        except:
            pass
        else:
            self.showWindow(SW_HIDE)

    def show(self):
        self.Me.deiconify()

    def hide(self):
        self.Me.withdraw()

    def showEx(self):
        try:
            from win32con import SW_SHOW
        except:
            pass
        else:
            self.showWindow(SW_SHOW)

    def removeCaption(self, isRemove: bool = True):
        self.Me.overrideredirect(isRemove)

    def removeCaptionEx(self):
        try:
            self.windows_manage.add_window_titlebar()
        except:
            pass

    def hideTitleBar(self):
        self.removeCaption()

    def hideTitleBarBorderEx(self):
        self.windows_manage.add_window_resize_border_frame()

    def hideTitleBarEx(self):
        self.removeCaptionEx()

    def mainLoop(self) -> None:
        prDebugging("Window -> MainLoop")
        try:
            self.Me.mainloop()
        except:
            prError("Window -> MainLoop -> Error")
        prDebugging("Window -> Quit")

    def run(self) -> None:
        prDebugging("Window -> Run")
        try:
            self.Me.mainloop()
        except:
            prError("Window -> Run -> Error")
        prDebugging("Window -> Quit")



# Widget
from Paperl.Paperui.Widgets.application import Application
from Paperl.Paperui.Widgets.window import Window
from Paperl.Paperui.Widgets.windowDevelop import WindowsDev, WindowsEffect, Windows22H2, Windows21H2
try:
    from Paperl.Paperui.Widgets.windowDevelop import ExButton
except:
    pass
from Paperl.Paperui.Widgets.toplevel import Toplevel
from Paperl.Paperui.Widgets.tooltip import Tooltip, TooltipEx
from Paperl.Paperui.Widgets.widget import Widget
from Paperl.Paperui.Widgets.button import Button
from Paperl.Paperui.Widgets.label import Label
from Paperl.Paperui.Widgets.message import Message
from Paperl.Paperui.Widgets.comboBox import ComboBox
from Paperl.Paperui.Widgets.checkButton import CheckButton
from Paperl.Paperui.Widgets.drag import Drag
from Paperl.Paperui.Widgets.scale import Scale
from Paperl.Paperui.Widgets.entry import Entry
from Paperl.Paperui.Widgets.sysTray import Systray
from Paperl.Paperui.Widgets.frame import Frame
from Paperl.Paperui.Widgets.text import Text
from Paperl.Paperui.Widgets.dragFrame import DragFrame
from Paperl.Paperui.Widgets.progressBar import ProgressBar
from Paperl.Paperui.Widgets.popupMenuEx import PopupMenuEx
from Paperl.Paperui.Widgets.childWindow import ChildWindowEx
from Paperl.Paperui.Widgets.optionMenu import OptionMenu
from Paperl.Paperui.Widgets.menu import Menu
from Paperl.Paperui.Widgets.headerBar import HeaderBarEx
from Paperl.Paperui.Widgets.sizeGrip import SizeGrip
from Paperl.Paperui.Widgets.spinBox import SpinBox
from Paperl.Paperui.Widgets.group import Group
from Paperl.Paperui.Widgets.menuButton import MenuButton

# Core
from Paperl.Paperui.Widgets.image import Image
from Paperl.Paperui.Widgets.eventHandle import EventHandle
from Paperl.Paperui.Widgets.windowDevelop import WindowsDev

# Often
from Paperl.Paperui.Widgets.constant import *
from Paperl.Paperui.Widgets.stringsVar import StringsVar
from Paperl.Paperui.Widgets.booleanVar import BooleanVar


def aboutPaperl(mica: bool = False):
    aboutDialog = Toplevel()
    aboutDialog.maximizeBox()
    aboutDialog.minimizeBox()
    aboutDialog.setTitle("")
    aboutDialog.setSize(700, 95)
    context = Label(aboutDialog, "Paperl 是一个高级图形用户界面开发库，使用tkinter开发，与各种依赖库的支持而打造出来的。"
                                 "@XiangQinxi 版权所有")
    context.pack(fillType=FILL_BOTH, expandType=EXPAND_YES, marginX=10, marginY=10)
    if mica:
        aboutDialog.useMica()
        context.setBackground("#000000")
        context.setForeground("#ffffff")
    return aboutDialog, context


from Paperl.Paperui.Widgets.widget import Widget


class Scale(Widget):
    def __init__(self, parent: Widget, startValue: float = 0, endValue: float = 100, value: float = 0, length: int = 100, orient="horizontal"):
        self.build(parent.Me, startValue, endValue, value, length, orient)

    def build(self, parent: Widget, startValue: float = 0, endValue: float = 100, value: float = 0, length: int = 100, orient="horizontal"):
        from tkinter.ttk import Scale
        self.Me = Scale(parent, from_=startValue, to=endValue, length=length, value=value, orient=orient)

    def onCommand(self, eventFunc: None = ...):
        self.Me.configure(command=eventFunc)

    def setOrient(self, orient):
        self.Me.configure(orient=orient)

    def getOrient(self):
        return self.Me.cget("orient")

    def setStartValue(self, startValue: float):
        self.Me.configure(from_=startValue)

    def getStartValue(self):
        return self.Me.cget("from_")

    def setEndValue(self, endValue: float):
        self.Me.configure(to=endValue)

    def getEndValue(self):
        return self.Me.cget("to")

    def setLength(self, length):
        self.Me.configure(length=length)

    def getLength(self):
        return self.Me.cget("length")

    def setValue(self, value):
        self.Me.configure(value=value)

    def getValue(self):
        return self.Me.cget("valur")


from Paperl.Paperc import prError


class Image(object):
    def __init__(self):
        self.build()

    def build(self):
        pass

    def setImageSvg(self, file):
        try:
            from tksvg import SvgImage
        except:
            prError("tkSvg -> Check -> Not Installed")
        else:
            self.Me = SvgImage(file=file)

    def setImage(self, file, height: int = None, width: int = None):
        from tkinter import PhotoImage
        if height is None and width is None:
            self.Me = PhotoImage(file=file)
        else:
            try:
                self.Me = PhotoImage(file=file, height=height, width=width)
            except:
                pass

    def export(self):
        try:
            return self.Me
        except:
            prError("Image -> export -> Error")


from Paperl.Paperui.Widgets.frame import Frame
from Paperl.Paperui.Widgets.widget import Widget


class DragFrame(Frame):
    def __init__(self, parent: Widget):
        self.build(parent.Me)

    def build(self, parent: Widget):
        from tkinter.ttk import Frame
        self.Me = Frame(parent)
        try:
            from tkdev4 import DevManage
        except:
            pass
        else:
            DevManage(self.Me).send_message_move_window(parent)


from Paperl.Paperui.Widgets.eventHandle import EventHandle
from Paperl.Paperc import prError, prDebugging, prWarring, prSuccess


class Widget(EventHandle):

    __name__ = "Widget"

    def __init__(self):
        super().__init__()

    def upDate(self):
        self.Me.update()

    def build(self):
        from tkinter import Widget
        try:
            self.Me = Widget()
        except:
            pass
        prDebugging("Widget -> Create")

    def setFocus(self):
        self.Me.focus()

    def setParent(self, parnet):
        try:
            from win32gui import SetParent
        except:
            pass
        else:
            try:
                SetParent(self.gethWnd(), parnet.gethWnd())
            except:
                pass

    def embedEx(self, widget):
        self.setParent(widget)

    def destroy(self):
        try:
            self.Me.destroy()
        except:
            pass

    def setState(self, state):
        from tkinter import TclError
        try:
            self.Me.configure(state=state)
        except TclError:
            prError("Widget -> State -> This property is not supported or this value is not supported")

    def setTearOff(self, tearOff: bool):
        from tkinter import TclError
        try:
            self.Me.configure(tearoff=tearOff)
        except TclError:
            prError("Widget -> TearOff -> This property is not supported or this value is not supported")

    def getTearOff(self):
        from tkinter import TclError
        try:
            self.getAttribute("tearoff")
        except TclError:
            prError("Widget -> TearOff -> This property is not supported or this value is not supported")

    def setMenu(self, menu):
        from tkinter import TclError
        try:
            self.Me.configure(menu=menu.Me)
        except TclError:
            prError("Widget -> Menu -> This property is not supported or this value is not supported")

    def getMenu(self):
        from tkinter import TclError
        try:
            self.getAttribute("menu")
        except TclError:
            prError("Widget -> Menu -> This property is not supported or this value is not supported")

    def setFont(self, family="", size: int = 12):
        from tkinter import TclError
        try:
            self.Me.configure(font=(family, size))
        except TclError:
            prError("Widget -> Font -> This property is not supported or this value is not supported")

    def tkCall(self, __command, *args):
        try:
            self.Me.tk.call(__command, args)
        except:
            prError("Widget -> Tcl -> Error")

    def useDpi(self):
        try:
            import ctypes
            ctypes.windll.shcore.SetProcessDpiAwareness(1)
            ScaleFactor = ctypes.windll.shcore.GetScaleFactorForDevice(0)
            self.Me.tk.call('tk', 'scaling', ScaleFactor / 75)
        except:
            prError("Widget -> Tcl -> Dpi -> The system does not support DPI")

    def setBackground(self, color):
        from tkinter import TclError
        try:
            self.Me.configure(background=color)
        except TclError:
            prError("Widget -> Background -> This property is not supported or this value is not supported")

    def setJustify(self, justifyType="left"):
        from tkinter import TclError
        try:
            self.Me.configure(justify=justifyType)
        except TclError:
            prError("Widget -> Justify -> This property is not supported or this value is not supported")

    def getJustify(self):
        from tkinter import TclError
        try:
            self.getAttribute("justify")
        except TclError:
            prError("Widget -> Justify -> This property is not supported or this value is not supported")

    def setAnchor(self, anchorType="center"):
        from tkinter import TclError
        try:
            self.Me.configure(anchor=anchorType)
        except TclError:
            prError("Widget -> Anchor -> This property is not supported or this value is not supported")

    def getAnchor(self):
        from tkinter import TclError
        try:
            self.getAttribute("anchor")
        except TclError:
            prError("Widget -> Anchor -> This property is not supported or this value is not supported")

    def setText(self, text: str):
        from tkinter import TclError
        try:
            self.Me.configure(text=text)
        except TclError:
            prError("Widget -> Text -> This property is not supported or this value is not supported")

    def getText(self):
        from tkinter import TclError
        try:
            return self.getAttribute("text")
        except TclError:
            prError("Widget -> Text -> This property is not supported or this value is not supported")

    def setImage(self, image):
        from tkinter import TclError
        try:
            self.Me.configure(image=image)
        except TclError:
            prError("Widget -> Image -> This property is not supported or this value is not supported")

    def getImage(self):
        from tkinter import TclError
        try:
            return self.getAttribute("image")
        except TclError:
            prError("Widget -> Image -> This property is not supported or this value is not supported")

    def getAttribute(self, attributeName: str):
        return self.Me.cget(attributeName)

    def setForeground(self, color):
        from tkinter import TclError
        try:
            self.Me.configure(foreground=color)
        except TclError:
            prError("Widget -> Foreground -> This property is not supported or this value is not supported")

    def getSize(self):
        return self.Me.winfo_width(), self.Me.winfo_height()

    def getId(self):
        return self.Me.winfo_id()

    def gethWnd(self):
        try:
            from win32gui import GetParent
        except:
            return None
        else:
            return GetParent(self.getId())

    def getSizeWidth(self) -> int:
        return self.Me.winfo_width()

    def getSizeWidth(self) -> int:
        return self.Me.winfo_height()

    def getPosition(self):
        return self.Me.winfo_x(), self.Me.winfo_y()

    def getPositionX(self) -> int:
        return self.Me.winfo_x()

    def getPositionY(self) -> int:
        return self.Me.winfo_y()

    def buttonUseDafaultStyle(self):
        self.Me.configure(style="TButton")

    def buttonUseSunValleyAccentStyle(self):
        try:
            self.Me.configure(style="Accent.TButton")
        except:
            prWarring(
                "Widget -> Use Accent Style -> Please use the SunValley theme"
            )

    def buttonUseSunValleyTitleBarStyle(self):
        try:
            self.Me.configure(style="Titlebar.TButton")
        except:
            prWarring(
                "Widget -> Use TitleBar Style -> Please use the SunValley theme"
            )

    def buttonUseSunValleyTitleBarCloseStyle(self):
        try:
            self.Me.configure(style="Close.Titlebar.TButton")
        except:
            prWarring(
                "Widget -> Use TitleBarClose Style -> Please use the SunValley theme"
            )

    def buttonUseSunValleyToggleStyle(self):
        try:
            self.Me.configure(style="Toggle.TButton")
        except:
            prWarring(
                "Widget -> Use Toggle Style -> Please use the SunValley theme"
            )

    def frameUseSunValleyCardStyle(self):
        try:
            self.Me.configure(style="Card.TFrame")
        except:
            prWarring(
                "Widget -> Use Card Style -> Please use the SunValley theme"
            )

    def checkButtonUseSunValleySwichStyle(self):
        try:
            self.Me.configure(style="Switch.TCheckbutton")
        except:
            prWarring(
                "Widget -> Use Swich Style -> Please use the SunValley theme"
            )

    def setStyle(self, style):
        try:
            self.Me.configure(style=style)
        except:
            prWarring(
                "Widget -> Set Style -> Cannot"
            )

    def pack(self, paddingX: int = None, paddingY: int = None,
             marginX: int = None, marginY: int = None,
             fillType=None, expandType=None, sideType=None, anchorType=None):
        try:
            self.Me.pack(ipadx=paddingX, ipady=paddingY, padx=marginX, pady=marginY,
                         fill=fillType, expand=expandType, side=sideType, anchor=anchorType)
        except:
            prError(
                "Widget -> Pack -> This component does not support this method or the value is filled in incorrectly"
            )

    def packForget(self):
        try:
            self.Me.pack_forget()
        except:
            prError(
                "Widget -> Pack Forget -> This component does not support this method or the value is filled in incorrectly"
            )

    def place(self, x: int = None, y: int = None, width: int = None, height: int = None, anchorType=None):
        try:
            self.Me.place(x=x, y=y, width=width, height=height, anchor=anchorType)
        except:
            prError(
                "Widget -> Place -> This component does not support this method or the value is filled in incorrectly"
            )

    def placeForget(self):
        try:
            self.Me.place_forget()
        except:
            prError(
                "Widget -> Place Forget -> This component does not support this method or the value is filled in incorrectly"
                    )

    def grid(self, column: int = None, row: int = None,
             paddingX: int = 0, paddingY: int = 0,
             marginX: int = 0, marginY: int = 0,):
        try:
            self.Me.grid(column=column, row=row, ipadx=paddingX, ipady=paddingY, padx=marginX, pady=marginY)
        except:
            prError(
                "Widget -> Grid -> This component does not support this method or the value is filled in incorrectly"
            )

    def gridForget(self):
        try:
            self.Me.grid_forget()
        except:
            prError(
                "Widget -> Grid Forget -> This component does not support this method or the value is filled in incorrectly"
                    )

    def showToast(self, title: str = "",
                  message: str = "Message",
                  appName: str = "Python",
                  appIcon: str = "",
                  timeOut: int = 0):
        try:
            from tkdev4 import DevNotification
        except:
            prError("tkDev4 -> Check -> Not Installed")
        else:
            try:
                DevNotification(title, message, appName, appIcon, timeOut).show()
            except:
                prError("Plyer -> Check -> Not Installed")


ANIMATION_LEFT_TO_RIGHT = "left-right"
ANIMATION_RIGHT_TO_LEFT = "right-left"
ANIMATION_TOP_TO_BOTTOM = "top-bottom"
ANIMATION_BOTTOM_TO_TOP = "bottom-top"

try:
    from win32con import AW_BLEND, AW_HIDE
    ANIMATION_WINDOWS_BLEND = AW_BLEND
    ANIMATION_WINDOWS_HIDE = AW_HIDE
except:
    pass


from Paperl.Paperui.Widgets.widget import Widget


class Button(Widget):
    def __init__(self, parent: Widget, text: str = ""):
        self.build(parent.Me, text)

    def build(self, parent, text: str):
        from tkinter.ttk import Button
        self.Me = Button(parent, text=text)

    def onCommand(self, eventFunc: None = ...):
        self.Me.configure(command=eventFunc)


from Paperl.Paperui.Widgets.widget import Widget


class Text(Widget):
    def __init__(self, parent: Widget):
        self.build(parent.Me)

    def build(self, parent):
        from tkinter import Text
        self.Me = Text(parent)
        self.setFont("微软雅黑", 9)
        self.Me.configure(border=0)

    def insertText(self, index="1.0", text: str = ""):
        self.Me.insert(index, text)

    def getText(self, start="1.0", end="end"):
        return self.Me.get(start, end)

    def getTextAll(self):
        return self.getText("1.0", "end")

    def deleteText(self, start="1.0", end="end"):
        self.Me.delete(start, end)

    def deleteTextAll(self):
        self.deleteText("1.0", "end")

    def setText(self, text: str):
        self.deleteTextAll()
        self.insertText(index="1.0", text=text)



from Paperl.Paperui.Widgets.dragFrame import DragFrame
from Paperl.Paperui.Widgets.widget import Widget
from Paperl.Paperui.Widgets.window import Window
from Paperl.Paperui.Widgets.label import Label
from Paperl.Paperui.Widgets.button import Button
from Paperl.Paperui.Widgets.sizeGrip import SizeGrip


class HeaderBarEx(DragFrame):
    def __init__(self, parent: Widget, window: Window, border: bool = True):
        super().__init__(parent)
        self.Window = window
        self.Window.setMinsize(100, 60)
        if border:
            self.Window.hideTitleBarEx()
        else:
            self.Window.hideTitleBar()

            try:
                self.Window.jumpTaskBar()
            except:
                pass

        self.Window.setExtendFrameIntoClientArea(0, 0, 0, 0)
        self.Title = Label(self)
        self.Close = Button(self)
        self.Max = Button(self)
        self.Min = Button(self)
        self.SizeGrip = SizeGrip(self.Window)
        self.isMin = False

    def maximize(self, event=None):
        if self.isMin:
            self.isMin = False

            self.Window.showNormalEx()
        else:
            self.isMin = True
            self.Window.showMaximizeEx()

    def setText(self, text: str):
        self.Window.setTitle(text)
        self.Title.setText(text)

    def setTitle(self, title: str):
        self.Window.setTitle(title)
        self.Title.setText(title)

    def addTitle(self, anchor="center"):
        self.Title.setFont("Microsoft YaHei UI", 9)
        self.Title.pack(sideType="left", anchorType=anchor, marginX=20, marginY=5)

    def addCloseButton(self, text: str = "×"):
        self.Close.setText(text)
        self.Close.onCommand(lambda: self.Window.destroy())
        self.Close.pack(sideType="right", fillType="y", marginX=10, marginY=5, paddingX=1)

    def addMinimizeButton(self, text: str = "–"):
        self.Min.setText(text)
        self.Min.onCommand(lambda: self.Window.showMinimizeEx())
        self.Min.pack(sideType="right", fillType="y", marginX=10, marginY=5, paddingX=1)

    def addMaximizeButton(self, text: str = "□"):
        self.Max.setText(text)
        self.Max.onCommand(self.maximize)
        self.Max.pack(sideType="right", fillType="y", marginX=10, marginY=5, paddingX=1)

    def useDoubleLeftMaximize(self):
        self.onButtonDoubleLeft(self.maximize)

    def useDoubleMiddleClose(self):
        self.onButtonDoubleMiddle(lambda event: self.Window.destroy())

    def addTitleBar(self):
        self.addCloseButton()
        self.addMaximizeButton()
        self.addMinimizeButton()
        self.addTitle()

    def useSunValley(self, theme="auto"):
        self.Window.useStyleSunValley(theme)
        self.Close.buttonUseSunValleyTitleBarCloseStyle()
        self.Max.buttonUseSunValleyTitleBarStyle()
        self.Min.buttonUseSunValleyTitleBarStyle()

    def addSizeGrip(self):
        self.SizeGrip.pack(sideType="bottom", anchorType="se", paddingX=5, paddingY=5)

    def show(self):
        self.pack(fillType="x", sideType="top")



from Paperl.Paperui.Widgets.constant import *
from Paperl.Paperc import prDebugging, prError


class EventHandle(object):
    def __init__(self):
        self.build()

    def build(self):
        from tkinter import Widget
        try:
            self.Me = Widget()
        except:
            pass

    def waitEvent(self, eventTime: int, eventFunc: None = ...):
        try:
            self.Me.after(eventTime, eventFunc)
        except:
            prError("Widget -> Bind -> Please confirm whether the eventTime is correct or the eventFunc is wrong")

    def bindEvent(self, eventName=None, eventFunc: None = ...):
        try:
            self.Me.bind(eventName, eventFunc)
        except:
            prError("Widget -> Bind -> Please confirm whether the eventName is correct or the eventFunc is wrong")

    # Event Button

    def onButtonLeft(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonLeft")
        self.bindEvent(EVENT_BUTTON1, eventFunc)

    def onButtonMiddle(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonMiddle")
        self.bindEvent(EVENT_BUTTON2, eventFunc)

    def onButtonRight(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonRight")
        self.bindEvent(EVENT_BUTTON3, eventFunc)

    def onButtonAll(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonAll")
        self.bindEvent(EVENT_BUTTON_ALL, eventFunc)

    def onButtonPress(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonPress")
        self.bindEvent(EVENT_BUTTON_PRESS, eventFunc)

    # Event Button Release

    def onButtonReleaseLeft(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonReleaseLeft")
        self.bindEvent(EVENT_BUTTON1_RELEASE, eventFunc)

    def onButtonReleaseMiddle(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonReleaseMiddle")
        self.bindEvent(EVENT_BUTTON2_RELEASE, eventFunc)

    def onButtonReleaseRight(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonReleaseRight")
        self.bindEvent(EVENT_BUTTON3_RELEASE, eventFunc)

    def onButtonReleaseAll(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonReleaseAll")
        self.bindEvent(EVENT_BUTTON_ALL_RELEASE, eventFunc)

    # Event Button Double

    def onButtonDoubleLeft(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonDoubleLeft")
        self.bindEvent(EVENT_BUTTON1_DOUBLE, eventFunc)

    def onButtonDoubleMiddle(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonDoubleMiddle")
        self.bindEvent(EVENT_BUTTON2_DOUBLE, eventFunc)

    def onButtonDoubleRight(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonDoubleRight")
        self.bindEvent(EVENT_BUTTON3_DOUBLE, eventFunc)

    def onButtonDoubleAll(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonDoubleAll")
        self.bindEvent(EVENT_BUTTON_ALL_DOUBLE, eventFunc)

    def onButtonDoublePress(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonDoublePress")
        self.bindEvent(EVENT_BUTTON_PRESS_DOUBLE, eventFunc)

    # Event Button Triple

    def onButtonTripleLeft(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonTripleLeft")
        self.bindEvent(EVENT_BUTTON1_TRIPLE, eventFunc)

    def onButtonTripleMiddle(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonTripleMiddle")
        self.bindEvent(EVENT_BUTTON2_TRIPLE, eventFunc)

    def onButtonTripleRight(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonTripleRight")
        self.bindEvent(EVENT_BUTTON3_TRIPLE, eventFunc)

    def onButtonTripleAll(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonTripleAll")
        self.bindEvent(EVENT_BUTTON_ALL_TRIPLE, eventFunc)

    def onButtonTriplePress(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onButtonTriplePress")
        self.bindEvent(EVENT_BUTTON_PRESS_TRIPLE, eventFunc)

    # Event Widget

    def onDestroy(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onDestroy")
        self.bindEvent(EVENT_DESTROY, eventFunc)

    def onKey(self, key: str = "", eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onKey")
        self.bindEvent(f"<{key}>", eventFunc)

    def onEnter(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onEnter")
        self.bindEvent(EVENT_ENTER, eventFunc)

    def onLeave(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onLeave")
        self.bindEvent(EVENT_LEAVE, eventFunc)

    def onFocusIn(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onFocusIn")
        self.bindEvent(EVENT_FOCUS_IN, eventFunc)

    def onFocusOut(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onFocusOut")
        self.bindEvent(EVENT_FOCUS_OUT, eventFunc)

    def onConfigure(self, eventFunc: None = ...):
        prDebugging("Widget -> Bind -> onConfigure")
        self.bindEvent(EVENT_CONFIGURE, eventFunc)



from Paperl.Paperc import prWarring


class Systray(object):
    def __init__(self, master):
        self.build(master.Me)

    def build(self, master):
        try:
            from tkdev4.devtcl import DevTray
        except:
            prError("tkDev4 -> tkdev4 needs to be installed")
        else:
            try:
                self.Me = DevTray(master)
            except:
                prWarring("tkdev4 -> Parameter error")


from tkinter import Canvas
from functools import partial

class SelectedCanvas(Canvas):

    def __init__(self, master=None, cnf={}, **kw):
        Canvas.__init__(self, master.Me, cnf, **kw)
        self.is_sizing = False
        self.old_width = 0
        self.old_height = 0
        self.old_pos_x = 0
        self.old_pos_y = 0
        self.start_x = 0
        self.start_y = 0
        self.start_root_x = 0
        self.start_root_y = 0
        self.on_resize_complete = None

    def set_on_resize_complete(self, on_resize_complete):
        self.on_resize_complete = on_resize_complete

    def on_update(self):
        """
        初始化后会被调用，在这里绘制矩形
        :return: None
        """
        self.create_rectangle(-1, -1, -2, -2, tag='side', dash=3, outline='red')
        for name in ('nw', 'w', 'sw', 'n', 's', 'ne', 'e', 'se'):
            self.create_rectangle(-1, -1, -2, -2, tag=name, outline='red')
            self.tag_bind(name, "<Enter>", partial(self.on_mouse_enter, name))
            self.tag_bind(name, "<Leave>", partial(self.on_mouse_leave, name))
            self.tag_bind(name, "<Button-1>", partial(self.on_mouse_click, name))
            self.tag_bind(name, "<B1-Motion>", partial(self.on_mouse_move, name))
            self.tag_bind(name, "<ButtonRelease-1>", partial(self.on_mouse_release, name))

    def show(self, is_fill=False):
        """
        显示
        :param is_fill: 是否填充
        :return: None
        """
        width = self.winfo_width()
        height = self.winfo_height()
        self.coords('side', 6, 6, width - 6, height - 6)
        self.coords('nw', 0, 0, 7, 7)
        self.coords('sw', 0, height - 8, 7, height - 1)
        self.coords('w', 0, (height - 7) / 2, 7, (height - 7) / 2 + 7)
        self.coords('n', (width - 7) / 2, 0, (width - 7) / 2 + 7, 7)
        self.coords('s', (width - 7) / 2, height - 8, (width - 7) / 2 + 7, height - 1)
        self.coords('ne', width - 8, 0, width - 1, 7)
        self.coords('se', width - 8, height - 8, width - 1, height - 1)
        self.coords('e', width - 8, (height - 7) / 2, width - 1, (height - 7) / 2 + 7)
        if is_fill:
            for name in ('nw', 'w', 'sw', 'n', 's', 'ne', 'e', 'se'):
                self.itemconfig(name, fill='red')

    def hide(self):
        """
        隐藏
        :return: None
        """
        self.coords('side', -1, -1, -2, -2,)
        for name in ('nw', 'w', 'sw', 'n', 's', 'ne', 'e', 'se'):
            self.coords(name, -1, -1, -2, -2)

    def on_mouse_enter(self, tag_name, event):
        """
        鼠标进入事件
        :param tag_name: 标签名字
        :param event: event
        :return: None
        """
        if tag_name in ("nw", "sw", "ne", "se"):
            self["cursor"] = "sizing"
        elif tag_name in ("w", "e"):
            self["cursor"] = "sb_h_double_arrow"
        else:
            self["cursor"] = "sb_v_double_arrow"

    def on_mouse_leave(self, tag_name, event):
        """
        鼠标离开事件
        :param tag_name: 标签名字
        :param event: event
        :return: None
        """
        if self.is_sizing:
            return
        self["cursor"] = "arrow"

    def on_mouse_click(self, tag_name, event):
        """
        鼠标点击事件
        :param tag_name: 标签名字
        :param event: event
        :return: None
        """
        self.is_sizing = True
        self.start_x = event.x
        self.start_y = event.y
        self.start_root_x = event.x_root
        self.start_root_y = event.y_root
        self.old_width = self.winfo_width()
        self.old_height = self.winfo_height()
        self.old_pos_x = int(self.place_info()['x'])
        self.old_pos_y = int(self.place_info()['y'])

    def on_mouse_move(self, tag_name, event):
        """
        鼠标移动事件
        :param tag_name: 标签名字
        :param event: event
        :return: None
        """
        if not self.is_sizing:
            return
        if 'e' in tag_name:
            width = max(0, self.old_width + (event.x - self.start_x))
            self.place_configure(width=width)
        if 'w' in tag_name:
            width = max(0, self.old_width + (self.start_root_x - event.x_root))
            to_x = event.x - self.start_x + int(self.place_info()['x'])
            self.place_configure(width=width, x=to_x)
        if 's' in tag_name:
            height = max(0, self.old_height + (event.y - self.start_y))
            self.place_configure(height=height)
        if 'n' in tag_name:
            height = max(0, self.old_height + (self.start_root_y - event.y_root))
            to_y = event.y - self.start_y + int(self.place_info()['y'])
            self.place_configure(height=height, y=to_y)
        self.after_idle(self.show)

    def on_mouse_release(self, tag_name, event):
        """
        鼠标松开事件
        :param tag_name: 标签名字
        :param event: event
        :return: None
        """
        self.is_sizing = False
        if self.on_resize_complete is not None:
            self.on_resize_complete()
        self["cursor"] = "arrow"



from Paperl.Paperui.Widgets.widget import Widget


class Balloon(Widget):
    def __init__(self, parent: Widget):
        self.build(parent.Me)

    def build(self, parent):
        try:
            from tkinter import tix
            self.Me = tix.Balloon(parent)
        except:
            pass

    def addWidget(self, widget: Widget, message: str = ""):
        self.Me.bind_widget(widget.Me, balloonmsg=message)


from Paperl.Paperui.Widgets.widget import Widget
from Paperl.Paperui.Widgets.toplevel import Toplevel


class PopupMenuEx(Toplevel):
    def __init__(self, parent: Widget):
        self.parent = parent
        self.build(parent.Me)

    def build(self, parent: Widget):
        from tkinter import Toplevel
        self.Me = Toplevel(parent)

        self.hide()
        self.removeCaptionEx()
        self.setRoundSmall()

        self.onFocusIn(lambda Evt: self.show())
        self.onFocusOut(lambda Evt: self.hide())

    def onCommand(self, eventName):
        self.bindEvent(eventName, self.popupMenu)

    def popupMenu(self, event):
        print("asda")
        self.setFocus()
        self.show()
        self.popup(event.x + self.parent.getPositionX(), event.y + self.parent.getPositionY() + 40)


from Paperl.Paperui.Widgets.widget import Widget


class SizeGrip(Widget):
    def __init__(self, parent: Widget):
        self.build(parent.Me)

    def build(self, parent):
        from tkinter.ttk import Sizegrip
        self.Me = Sizegrip(parent)



from Paperl.Paperui.Widgets.widget import Widget


class AnimationWindows(object):
    def __init__(self, parent: Widget, timeMs=200, flags=0x00080000):
        self.build(parent.gethWnd(), timeMs, flags)

    def build(self, parent: Widget, timeMs, flags):
        from ctypes import windll
        from ctypes import c_int
        self.Me = windll.user32.AnimateWindow(parent, timeMs, flags)
        return self.Me